@@ -18,7 +18,9 @@
 #
 # Refer to the README and COPYING files for full details of the license
 #
+
 from __future__ import absolute_import
+from __future__ import division
 
 from contextlib import contextmanager
 import logging
@@ -27,15 +29,15 @@ import threading
 import libvirt
 
 from vdsm import constants
-from vdsm import containersconnection
 from vdsm.common import cpuarch
 from vdsm.common import libvirtconnection
 from vdsm.common import response
 import vdsm.common.time
-from vdsm.network import ipwrapper
+from vdsm.common import xmlutils
 from vdsm.virt import sampling
 from vdsm.virt import vm
 from vdsm.virt.domain_descriptor import DomainDescriptor
+from vdsm.virt.vmdevices import core
 
 from testlib import namedTemporaryDir
 from testlib import recorded
@@ -110,6 +112,7 @@ class ClientIF(object):
         self.vmRequests = {}
         self.bindings = {}
         self._recovery = False
+        self.unknown_vm_ids = []
 
     def createVm(self, vmParams, vmRecover=False):
         self.vmRequests[vmParams['vmId']] = (vmParams, vmRecover)
@@ -131,6 +134,11 @@ class ClientIF(object):
         with self.vmContainerLock:
             return self.vmContainer.copy()
 
+    def pop_unknown_vm_ids(self):
+        ret = self.unknown_vm_ids
+        self.unknown_vm_ids = []
+        return ret
+
 
 class Domain(object):
     def __init__(self, xml='',
@@ -138,37 +146,39 @@ class Domain(object):
                  errorMessage="",
                  domState=libvirt.VIR_DOMAIN_RUNNING,
                  domReason=0,
-                 vmId=''):
+                 vmId='', vm=None):
         self._xml = xml
         self.devXml = ''
-        self._virtError = virtError
+        self.virtError = virtError
         self._errorMessage = errorMessage
         self._metadata = ""
         self._io_tune = {}
-        self._domState = domState
-        self._domReason = domReason
+        self.domState = domState
+        self.domReason = domReason
         self._vmId = vmId
+        self.vm = vm
         self._diskErrors = {}
         self._downtimes = []
+        self.destroyed = False
 
     @property
     def connected(self):
         return True
 
     def _failIfRequested(self):
-        if self._virtError != libvirt.VIR_ERR_OK:
-            raise Error(self._virtError, self._errorMessage)
+        if self.virtError != libvirt.VIR_ERR_OK:
+            raise Error(self.virtError, self._errorMessage)
 
     def UUIDString(self):
         return self._vmId
 
     def state(self, unused):
         self._failIfRequested()
-        return (self._domState, self._domReason)
+        return (self.domState, self.domReason)
 
     def info(self):
         self._failIfRequested()
-        return (self._domState, )
+        return (self.domState, )
 
     def XMLDesc(self, unused):
         return self._xml
@@ -246,16 +256,19 @@ class Domain(object):
         }
 
     def destroy(self):
+        self.destroyed = True
+
+    def undefineFlags(self, flags=0):
         pass
 
-    def undefine(self):
+    def attachDevice(self, device_xml):
         pass
 
-    def attachDevice(self, device):
-        pass
-
-    def detachDevice(self, device):
-        pass
+    def detachDevice(self, device_xml):
+        if self.vm is not None:
+            dev = xmlutils.fromstring(device_xml)
+            alias = core.find_device_alias(dev)
+            self.vm.onDeviceRemoved(alias)
 
 
 class GuestAgent(object):
@@ -295,7 +308,6 @@ def VM(params=None, devices=None, runCpu=False,
     with namedTemporaryDir() as tmpDir:
         with MonkeyPatchScope([(constants, 'P_VDSM_RUN', tmpDir),
                                (libvirtconnection, 'get', Connection),
-                               (containersconnection, 'get', Connection),
                                ]):
             if params and 'xml' in params:
                 vmParams = {}
@@ -310,7 +322,6 @@ def VM(params=None, devices=None, runCpu=False,
             fake._update_metadata = lambda: None
             fake._sync_metadata = lambda: None
             fake.send_status_event = lambda **kwargs: None
-            fake._waitForDeviceRemoval = lambda device: None
             fake.arch = arch
             fake.guestAgent = GuestAgent()
             fake.conf['devices'] = [] if devices is None else devices
@@ -405,8 +416,6 @@ class HostSample(object):
     def __init__(self, timestamp, samples):
         self.timestamp = timestamp
         self.cpuCores = CpuCoreSample(samples)
-        self.interfaces = {
-            'lo': sampling.InterfaceSample(ipwrapper.getLink('lo'))}
 
 
 CREATED = "created"
